//Software base para OPS150
//Controlador ESP32
//Mapeamento dos IOs e definição das Seriais
// Definição dos pinos do controlador

//A blibioteca Wire é utilizada para acesso a expansão de memória EEPROM
//Trata-se de uma biblioteca padrão do Arduino
#include <Wire.h>

//Entradas digitais
//O OPS150 utiliza um módulo de expansão para aumentar o número de entradas digitais
//Devido a isso as entradas X1 a X8 não possuem pinos dedicados para leitura
//A varredura de leitura das entradas é apresentada na função loop
#define pin_Latch165 15
#define pin_Clock165 4
#define pin_Data165 5
#define pin_X9 23
#define pin_X10 35

//Entradas analógicas
#define pin_AI1 34
#define pin_AI2 39
#define pin_AI3 36

//Saídas digitais
//O OPS150 utiliza um módulo de expansão para aumentar o número de saídas digitais
//Devido a isso as saídas Y1 a Y8 não possuem pinos dedicados para escrita
//A varredura de acionamento das saídas é apresentada na função loop
#define pin_Latch595 14
#define pin_Clock595 27
#define pin_Data595 13
#define pin_Enable595 12
#define pin_Y9 26
#define pin_Y10 25

//Saídas analógicas
#define pin_AO1 33
#define pin_AO2 32

//Memoria externa
#define pin_E2P_SDA 19
#define pin_E2P_SCL 18

//Pinos auxiliares
#define ENTX_pin 2 //Pino de comunicação RS485

//Variaveis para controle das entradas e saidas digitais
bool Digital_Out[10];
bool Digital_In[10];
int Analog_In[3];
int Analog_Out[2];

//Variaveis para E2P
#define diviceAddress 80
unsigned int address = 0;
byte Data_byte = 0;

#include "EEPROM_Ctrl.h"

void setup() {
  Serial.begin(115200);//USB
  Serial1.begin(115200);//TTL/RS232
  Serial2.begin(115200);//RS485

  //Inicializa hardware
  //Entradas
  pinMode(pin_Latch165, OUTPUT);
  pinMode(pin_Clock165, OUTPUT);
  pinMode(pin_Data165, INPUT);
  //Saídas
  pinMode(pin_Latch595, OUTPUT);
  pinMode(pin_Clock595, OUTPUT);
  pinMode(pin_Data595, OUTPUT);
  pinMode(pin_Enable595, OUTPUT);
  //IO rápido
  pinMode(pin_X9, INPUT);
  pinMode(pin_X10, INPUT);
  pinMode(pin_Y9, OUTPUT);
  pinMode(pin_Y10, OUTPUT);
  //485 entx
  pinMode(ENTX_pin, OUTPUT);

  //Inicia saídas em LOW
   digitalWrite(pin_Latch595, LOW);
  for (int i = 7; i >= 0; i--) {
    digitalWrite(pin_Data595, LOW);
    digitalWrite(pin_Clock595, HIGH);
    digitalWrite(pin_Clock595, LOW);
  }
  digitalWrite(pin_Latch595, HIGH);
  digitalWrite(pin_Enable595, HIGH);
  digitalWrite(pin_Y9, LOW);
  digitalWrite(pin_Y10, LOW);

  //definições função PWM para as sídas analógicas
  ledcSetup(0, 1000, 12);
  ledcAttachPin(pin_AO1, 0);
  ledcSetup(1, 1000, 12);
  ledcAttachPin(pin_AO2, 1);

  //incio da biblioteca I2C para acesso a memória EEPROM
  Wire.begin(pin_E2P_SDA, pin_E2P_SCL);

}

void loop() {
  //************************************************************************************
  //Leitura das entradas digitais
  //Esse código deve rodar sempre para leitura das entradas digitais
  //Utilizar o Array Digital_In[0] a Digital_In[9] no software para controle
  //************************************************************************************
  digitalWrite(pin_Latch165, LOW);
  digitalWrite(pin_Latch165, HIGH); 
  for (int i = 0; i <= 7; i++){
    Digital_In[i] = !digitalRead(pin_Data165);
    digitalWrite(pin_Clock165, HIGH);
    digitalWrite(pin_Clock165, LOW);
  }
  Digital_In[8] = !digitalRead(pin_X9);
  Digital_In[9] = !digitalRead(pin_X10);

  //************************************************************************************
  //Escrita das saídas digitais
  //Esse código deve rodar sempre para escrita das saidas digitais
  //Utilizar o Array Digital_Out[0] a Digital_Out[9] no software para controle
  //************************************************************************************
  digitalWrite(pin_Latch595, LOW);
  for (int i = 7; i >= 0; i--) {
    digitalWrite(pin_Data595, Digital_Out[i]);
    digitalWrite(pin_Clock595, HIGH);
    digitalWrite(pin_Clock595, LOW);
  }
  digitalWrite(pin_Latch595, HIGH);
  digitalWrite(pin_Y9, Digital_Out[8]);
  digitalWrite(pin_Y10, Digital_Out[9]);

  //************************************************************************************
  //Leitura das entradas Analógicas
  //Esse código deve rodar sempre para leitura das entradas analógicas
  //Utilizar o Array Analog_In[0] a Analog_In[2] no software para controle
  //************************************************************************************
  Analog_In[0] = analogRead(pin_AI1);
  Analog_In[1] = analogRead(pin_AI2);
  Analog_In[2] = analogRead(pin_AI3);

  //************************************************************************************
  //Escrita das saídas Analógicas / 0 >= dutyCycle <= 3722 (0-10V)
  //Esse código deve rodar sempre para escrita das saidas analógicas
  //Utilizar o Array Analog_Out[0] a Analog_Out[1] no software para controle
  //************************************************************************************
  ledcWrite(0, Analog_Out[0]);
  ledcWrite(1, Analog_Out[1]);

  //************************************************************************************
  //Acesso a memória EEPROM
  //Cada posição de memória tem 1 byte
  //Cada página da memória tem 128 bytes
  //************************************************************************************
  //writeEEPROM(diviceAddress, address, Data_byte);//Não está habilitado para evitar desgaste
  Data_byte = readEEPROM(diviceAddress, address);

}
